@@ -1,7 +1,9 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import unicodedata
+
 from collections import defaultdict
+from dateutil.relativedelta import relativedelta
 from decimal import Decimal
 from operator import attrgetter
 
@@ -15,7 +17,7 @@ from trytond.i18n import gettext
 from trytond.model import ModelSQL, ModelView, fields
 from trytond.model.modelsql import convert_from
 from trytond.pool import Pool
-from trytond.pyson import Eval
+from trytond.pyson import Eval, If
 from trytond.report import Report
 from trytond.transaction import Transaction
 from trytond.wizard import Wizard, StateView, StateTransition, StateReport, \
@@ -24,11 +26,6 @@ from trytond.modules.account_eu.account import ECSalesList, ECSalesListContext
 
 from .exceptions import PrintError
 
-__all__ = ['AEAT111', 'AEAT115', 'AEAT303', 'PrintAEATStart', 'PrintAEAT',
-    'ESVATList', 'ESVATListContext', 'AEAT347', 'ECOperationList',
-    'ECOperationListContext', 'AEAT349']
-
-
 # XXX fix: https://genshi.edgewall.org/ticket/582
 from genshi.template.astutil import ASTCodeGenerator, ASTTransformer
 if not hasattr(ASTCodeGenerator, 'visit_NameConstant'):
@@ -291,9 +288,13 @@ class AEAT303(AEATReport):
 
         amount_to_compensate = Decimal(0)
         fiscalyear = periods[0].fiscalyear
+        start_periods = Period.search([
+                ('fiscalyear', '=', fiscalyear.id),
+                ('end_date', '<=', start_date),
+                ])
         with transaction.set_context({
                     'fiscalyear': fiscalyear.id,
-                    'to_date': end_date,
+                    'periods': [p.id for p in start_periods],
                     }):
             for account in Account.search([
                         ('company', '=', fiscalyear.company.id),
@@ -415,22 +416,38 @@ class ESVATList(ModelSQL, ModelView):
     def search_province_code(cls, name, clause):
         return [(('party.es_province_code',) + tuple(clause[1:]))]
 
+    @classmethod
+    def excluded_tax_codes(cls):
+        return ['111', '115']
+
     @classmethod
     def table_query(cls):
         pool = Pool()
         Invoice = pool.get('account.invoice')
         InvoiceTax = pool.get('account.invoice.tax')
         Tax = pool.get('account.tax')
+        TaxCode = pool.get('account.tax.code')
+        TaxCodeLine = pool.get('account.tax.code.line')
         Date = pool.get('ir.date')
         context = Transaction().context
         invoice = Invoice.__table__()
         cancel_invoice = Invoice.__table__()
         invoice_tax = InvoiceTax.__table__()
         tax = Tax.__table__()
+        tax_code = TaxCode.__table__()
+        tax_code_line = TaxCodeLine.__table__()
+        exclude_invoice_tax = InvoiceTax.__table__()
 
         amount = invoice_tax.base + invoice_tax.amount
         month = Extract('MONTH', invoice.invoice_date)
 
+        excluded_taxes = (tax_code_line
+            .join(tax_code,
+                condition=(tax_code.id == tax_code_line.code)
+                ).select(
+                    tax_code_line.tax, distinct=True,
+                    where=tax_code.aeat_report.in_(cls.excluded_tax_codes())))
+
         where = ((invoice.company == context.get('company'))
             & (invoice.state.in_(['posted', 'paid']))
             & (tax.es_vat_list_code != Null)
@@ -438,7 +455,12 @@ class ESVATList(ModelSQL, ModelView):
                 == context.get('date', Date.today()).year)
             & ~Exists(cancel_invoice.select(
                     cancel_invoice.cancel_move, distinct=True,
-                    where=(cancel_invoice.cancel_move == invoice.move))))
+                    where=(cancel_invoice.cancel_move == invoice.move)))
+            # Use exists to exclude the full invoice when it has multiple taxes
+            & ~Exists(exclude_invoice_tax.select(
+                    exclude_invoice_tax.invoice,
+                    where=((exclude_invoice_tax.invoice == invoice_tax.invoice)
+                        & (exclude_invoice_tax.tax.in_(excluded_taxes))))))
         return (invoice_tax
             .join(invoice,
                 condition=invoice_tax.invoice == invoice.id)
@@ -550,10 +572,11 @@ class ECOperationList(ECSalesList):
 
         sales = super().table_query()
 
-        where = ((invoice.company == context.get('company'))
-            & (period.fiscalyear == context.get('fiscalyear')))
-        if context.get('period'):
-            where &= (period.id == context.get('period'))
+        where = invoice.company == context.get('company')
+        if context.get('start_date'):
+            where &= (move.date >= context.get('start_date'))
+        if context.get('end_date'):
+            where &= (move.date <= context.get('end_date'))
         where &= ((tax.es_ec_purchases_list_code != Null)
             & (tax.es_ec_purchases_list_code != ''))
         where &= invoice.type == 'in'
@@ -590,6 +613,35 @@ class ECOperationListContext(ECSalesListContext):
     "EC Operation List Context"
     __name__ = 'account.reporting.es_ec_operation_list.context'
 
+    start_date = fields.Date("Start Date",
+        domain=[
+            If(Eval('end_date'),
+                ('start_date', '<=', Eval('end_date')),
+                (),
+                ),
+            ],
+        depends=['end_date'])
+    end_date = fields.Date("End Date",
+        domain=[
+            If(Eval('start_date'),
+                ('end_date', '>=', Eval('start_date')),
+                (),
+                ),
+            ],
+        depends=['start_date'])
+
+    @classmethod
+    def default_start_date(cls):
+        pool = Pool()
+        Date = pool.get('ir.date')
+        return Date.today() - relativedelta(months=1, day=1)
+
+    @classmethod
+    def default_end_date(cls):
+        pool = Pool()
+        Date = pool.get('ir.date')
+        return Date.today() - relativedelta(months=1, day=31)
+
 
 class AEAT349(Report):
     __name__ = 'account.reporting.aeat349'
@@ -597,27 +649,23 @@ class AEAT349(Report):
     @classmethod
     def get_context(cls, records, data):
         pool = Pool()
-        Period = pool.get('account.period')
-        Fiscalyear = pool.get('account.fiscalyear')
+        Company = pool.get('company.company')
         t_context = Transaction().context
 
         context = super().get_context(records, data)
 
-        fiscalyear = Fiscalyear(t_context['fiscalyear'])
-        context['year'] = str(fiscalyear.start_date.year)
-        context['company'] = fiscalyear.company
+        context['company'] = Company(t_context['company'])
         context['records_amount'] = sum(
             (r.amount for r in records), Decimal(0))
 
-        period_id = t_context.get('period')
-        if not period_id:
-            # Yearly
-            context['period'] = '0A'
-            context['period_number'] = '99'
-        else:
-            period = Period(period_id)
-            start_month = period.start_date.month
-            end_month = period.end_date.month
+        start_date = t_context.get('start_date')
+        end_date = t_context.get('end_date')
+        if start_date or end_date:
+            date = start_date or end_date
+            context['year'] = str(date.year)
+        if start_date and end_date:
+            start_month = start_date.month
+            end_month = end_date.month
             if end_month - start_month > 0:
                 context['period'] = str(end_month // 3) + 'T'
                 context['period_number'] = str(20 + (end_month // 3))
